"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const assert = require('assert');

const {
  HttpStatusError
} = require('common-errors');

const attach = require('./attach');

const providers = require('../providers');

const {
  getSignedToken
} = require('./get-signed-token');
/**
 * @param {Service} ctx
 * @param {ServiceRequest} request
 * @param {{ account: any, jwt: string, user: any }} credentials
 */


async function formOAuthResponse(ctx, request, credentials) {
  const {
    account,
    jwt,
    user
  } = credentials; // logged in, no account provided - bypass

  if (!account && jwt) {
    const {
      cookies
    } = ctx.config.jwt;

    if (cookies.enabled === true) {
      request.transportRequest.setState(cookies.name, jwt, cookies.settings);
    }

    return {
      payload: _objectSpread({}, credentials),
      error: false,
      type: 'ms-users:logged-in',
      title: 'signing in'
    };
  }

  if (!account) {
    throw new HttpStatusError(500, 'no account when jwt isn\'t present');
  }

  const {
    missingPermissions
  } = account;
  const Provider = providers[account.provider];
  assert(Provider, `${account.provider} is not usable`);
  const accountResponse = Provider.transformAccountToResponseFormat(account);
  const context = user ? attach.call(ctx, accountResponse, user) : getSignedToken.call(ctx, accountResponse);
  return {
    payload: await context,
    error: false,
    missingPermissions,
    type: user ? 'ms-users:attached' : 'ms-users:signed',
    title: `Attached ${account.provider} account`
  };
}

module.exports = formOAuthResponse;