"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const partial = require('lodash/partial');

const moment = require('moment');

const generateEmail = require('./email/generate.js');

const {
  CHALLENGE_TYPE_EMAIL,
  CHALLENGE_TYPE_PHONE
} = require('../../constants.js');

const DetailedHttpStatusError = require('../detailed-error');

const sendSms = require('./phone/send'); // contains challenges


const CHALLENGES = {
  [CHALLENGE_TYPE_EMAIL]: (action, ctx, wait = false) => partial(generateEmail, partial.placeholder, action, ctx, {
    wait,
    send: true
  }),
  [CHALLENGE_TYPE_PHONE]: (action, ctx) => partial(sendSms, partial.placeholder, action, ctx)
}; // select challenge helper

const selectChallenge = (type, action, ctx, wait) => CHALLENGES[type](action, ctx, wait);
/**
 * Send an email with appropriate content
 *
 * @param {String=CHALLENGE_TYPE_EMAIL,CHALLENGE_TYPE_PHONE} type
 * @param {Object} opts - `ms-token` options
 * @param {Object} ctx - additional context
 * @param {Boolean} [wait=false] - should we wait for challenge confirmation?
 * @return {Promise}
 */


async function generateChallenge(type, opts, ctx = {}, wait = false) {
  try {
    const token = await this.tokenManager.create(opts);
    ctx.token = token;
  } catch (error) {
    if (error.message === '429') {
      const {
        mailto: email
      } = this.config.validation;
      const duration = moment().add(opts.throttle, 'seconds').toNow(true);
      const msg = `We've already sent you an email, if it doesn't come - please try again in ${duration} or send us an email`;

      const reason = _objectSpread(_objectSpread({}, error.reason), {}, {
        duration: opts.throttle,
        email
      });

      throw DetailedHttpStatusError(429, msg, reason);
    }

    throw error;
  }

  const handler = selectChallenge(type, opts.action, ctx, wait);
  return handler.call(this, opts.id);
}

generateChallenge.selectChallenge = selectChallenge;
module.exports = generateChallenge;